@@ -73,7 +73,7 @@ static unsigned stringToColour(const ::WPXString &s)
   return val;
 }
 
-libvisio::VSDSVGGenerator::VSDSVGGenerator(std::ostream &output_sink): m_gradient(), m_style(), m_gradientIndex(1), m_shadowIndex(1), m_isFirstPage(true), m_outputSink(output_sink)
+libvisio::VSDSVGGenerator::VSDSVGGenerator(libvisio::VSDStringVector &vec): m_gradient(), m_style(), m_gradientIndex(1), m_patternIndex(1), m_shadowIndex(1), m_outputSink(), m_vec(vec)
 {
 }
 
@@ -83,17 +83,6 @@ libvisio::VSDSVGGenerator::~VSDSVGGenerator()
 
 void libvisio::VSDSVGGenerator::startGraphics(const WPXPropertyList &propList)
 {
-  if (m_isFirstPage)
-    m_isFirstPage = false;
-  else
-    m_outputSink << "<hr/>\n";
-
-  m_outputSink << "<!-- \n";
-  m_outputSink << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n";
-  m_outputSink << "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\"";
-  m_outputSink << " \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n";
-  m_outputSink << " -->\n";
-
   m_outputSink << "<svg:svg version=\"1.1\" xmlns:svg=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" ";
   if (propList["svg:width"])
     m_outputSink << "width=\"" << doubleToString(72*(propList["svg:width"]->getDouble())) << "\" ";
@@ -105,6 +94,8 @@ void libvisio::VSDSVGGenerator::startGraphics(const WPXPropertyList &propList)
 void libvisio::VSDSVGGenerator::endGraphics()
 {
   m_outputSink << "</svg:svg>\n";
+  m_vec.append(m_outputSink.str().c_str());
+  m_outputSink.str("");
 }
 
 void libvisio::VSDSVGGenerator::setStyle(const ::WPXPropertyList &propList, const ::WPXPropertyListVector &gradient)
@@ -297,6 +288,50 @@ void libvisio::VSDSVGGenerator::setStyle(const ::WPXPropertyList &propList, cons
       }
     }
   }
+  else if(m_style["draw:fill"] && m_style["draw:fill"]->getStr() == "bitmap")
+  {
+    if (m_style["draw:fill-image"] && m_style["libwpg:mime-type"])
+    {
+      m_outputSink << "<svg:defs>\n";
+      m_outputSink << "  <svg:pattern id=\"img" << m_patternIndex++ << "\" patternUnits=\"userSpaceOnUse\" ";
+      if (m_style["svg:width"])
+        m_outputSink << "width=\"" << doubleToString(72*(m_style["svg:width"]->getDouble())) << "\" ";
+      else
+        m_outputSink << "width=\"100\" ";
+
+      if (m_style["svg:height"])
+        m_outputSink << "height=\"" << doubleToString(72*(m_style["svg:height"]->getDouble())) << "\">" << std::endl;
+      else
+        m_outputSink << "height=\"100\">" << std::endl;
+      m_outputSink << "<svg:image ";
+
+      if (m_style["svg:x"])
+        m_outputSink << "x=\"" << doubleToString(72*(m_style["svg:x"]->getDouble())) << "\" ";
+      else
+        m_outputSink << "x=\"0\" ";
+
+      if (m_style["svg:y"])
+        m_outputSink << "y=\"" << doubleToString(72*(m_style["svg:y"]->getDouble())) << "\" ";
+      else
+        m_outputSink << "y=\"0\" ";
+
+      if (m_style["svg:width"])
+        m_outputSink << "width=\"" << doubleToString(72*(m_style["svg:width"]->getDouble())) << "\" ";
+      else
+        m_outputSink << "width=\"100\" ";
+
+      if (m_style["svg:height"])
+        m_outputSink << "height=\"" << doubleToString(72*(m_style["svg:height"]->getDouble())) << "\" ";
+      else
+        m_outputSink << "height=\"100\" ";
+
+      m_outputSink << "xlink:href=\"data:" << m_style["libwpg:mime-type"]->getStr().cstr() << ";base64,";
+      m_outputSink << m_style["draw:fill-image"]->getStr().cstr();
+      m_outputSink << "\" />\n";
+      m_outputSink << "  </svg:pattern>\n";
+      m_outputSink << "</svg:defs>\n";
+    }
+  }
 }
 
 void libvisio::VSDSVGGenerator::startLayer(const ::WPXPropertyList &propList)
@@ -625,6 +660,9 @@ void libvisio::VSDSVGGenerator::writeStyle(bool /* isClosed */)
   if(m_style["draw:fill"] && m_style["draw:fill"]->getStr() == "gradient")
     m_outputSink << "fill: url(#grad" << m_gradientIndex-1 << "); ";
 
+  if(m_style["draw:fill"] && m_style["draw:fill"]->getStr() == "bitmap")
+    m_outputSink << "fill: url(#img" << m_patternIndex-1 << "); ";
+
   if(m_style["draw:shadow"] && m_style["draw:shadow"]->getStr() == "visible")
     m_outputSink << "filter:url(#shadow" << m_shadowIndex-1 << "); ";
 
