@@ -243,35 +243,8 @@ void libvisio::VSDXContentCollector::_fillAndShadowProperties(unsigned colourInd
   m_styleProps.insert("draw:fill", m_fillType);
 }
 
-void libvisio::VSDXContentCollector::_lineProperties(double strokeWidth, Colour c, unsigned linePattern, unsigned startMarker, unsigned endMarker, unsigned lineCap)
+void libvisio::VSDXContentCollector::_applyLinePattern()
 {
-  m_linePattern = linePattern;
-
-  if (linePattern == 0) return; // No need to add style
-
-  m_styleProps.insert("svg:stroke-width", m_scale*strokeWidth);
-  m_lineColour = getColourString(c);
-  m_styleProps.insert("svg:stroke-color", m_lineColour);
-  if (c.a)
-    m_styleProps.insert("svg:stroke-opacity", (1 - c.a/255.0), WPX_PERCENT);
-  else
-    m_styleProps.insert("svg:stroke-opacity", 1.0, WPX_PERCENT);
-  switch (lineCap)
-  {
-  case 0:
-    m_styleProps.insert("svg:stroke-linecap", "round");
-    m_styleProps.insert("svg:stroke-linejoin", "round");
-    break;
-  case 2:
-    m_styleProps.insert("svg:stroke-linecap", "square");
-    m_styleProps.insert("svg:stroke-linejoin", "miter");
-    break;
-  default:
-    m_styleProps.insert("svg:stroke-linecap", "butt");
-    m_styleProps.insert("svg:stroke-linejoin", "miter");
-    break;
-  }
-
   int dots1 = 0;
   int dots2 = 0;
   double dots1len = 0.0;
@@ -279,7 +252,7 @@ void libvisio::VSDXContentCollector::_lineProperties(double strokeWidth, Colour
   double gap = 0.0;
 
   m_styleProps.remove("draw:stroke");
-  switch (linePattern)
+  switch (m_linePattern)
   {
   case 2: // "6, 3"
     dots1 = dots2 = 1;
@@ -425,11 +398,11 @@ void libvisio::VSDXContentCollector::_lineProperties(double strokeWidth, Colour
     break;
   }
 
-  if (linePattern == 0)
+  if (m_linePattern == 0)
     m_styleProps.insert("draw:stroke", "none");
-  else if (linePattern == 1)
+  else if (m_linePattern == 1)
     m_styleProps.insert("draw:stroke", "solid");
-  else if (linePattern > 1 && linePattern <= 23)
+  else if (m_linePattern > 1 && m_linePattern <= 23)
   {
     m_styleProps.insert("draw:stroke", "dash");
     m_styleProps.insert("draw:dots1", dots1);
@@ -442,19 +415,155 @@ void libvisio::VSDXContentCollector::_lineProperties(double strokeWidth, Colour
     // FIXME: later it will require special treatment for custom line patterns
     // patt ID is 0xfe, link to stencil name is in 'Line' blocks
     m_styleProps.insert("draw:stroke", "solid");
+}
+
+void libvisio::VSDXContentCollector::_lineProperties(double strokeWidth, Colour c, unsigned linePattern, unsigned startMarker, unsigned endMarker, unsigned lineCap)
+{
+  m_linePattern = linePattern;
+
+  if (linePattern == 0) return; // No need to add style
+
+  m_styleProps.insert("svg:stroke-width", m_scale*strokeWidth);
+  m_lineColour = getColourString(c);
+  m_styleProps.insert("svg:stroke-color", m_lineColour);
+  if (c.a)
+    m_styleProps.insert("svg:stroke-opacity", (1 - c.a/255.0), WPX_PERCENT);
+  else
+    m_styleProps.insert("svg:stroke-opacity", 1.0, WPX_PERCENT);
+  switch (lineCap)
+  {
+  case 0:
+    m_styleProps.insert("svg:stroke-linecap", "round");
+    m_styleProps.insert("svg:stroke-linejoin", "round");
+    break;
+  case 2:
+    m_styleProps.insert("svg:stroke-linecap", "square");
+    m_styleProps.insert("svg:stroke-linejoin", "miter");
+    break;
+  default:
+    m_styleProps.insert("svg:stroke-linecap", "butt");
+    m_styleProps.insert("svg:stroke-linejoin", "miter");
+    break;
+  }
+
+  _applyLinePattern();
 
   // Deal with line markers (arrows, etc.)
   if (startMarker > 0)
   {
-    m_styleProps.insert("draw:marker-start-viewbox", "0 0 20 30");
-    m_styleProps.insert("draw:marker-start-path", "m10 0-10 30h20z");
-    m_styleProps.insert("draw:marker-start-width", m_scale*0.118);
+    m_styleProps.insert("draw:marker-start-viewbox", _linePropertiesMarkerViewbox(startMarker));
+    m_styleProps.insert("draw:marker-start-path", _linePropertiesMarkerPath(startMarker));
+    m_styleProps.insert("draw:marker-start-width", m_scale*_linePropertiesMarkerScale(startMarker)*(0.1/(strokeWidth*strokeWidth+1)+2.54*strokeWidth));
   }
-  else if (endMarker > 0)
+  if (endMarker > 0)
   {
-    m_styleProps.insert("draw:marker-end-viewbox", "0 0 20 30");
-    m_styleProps.insert("draw:marker-end-path", "m10 0-10 30h20z");
-    m_styleProps.insert("draw:marker-end-width", m_scale*0.118);
+    m_styleProps.insert("draw:marker-end-viewbox", _linePropertiesMarkerViewbox(endMarker));
+    m_styleProps.insert("draw:marker-end-path", _linePropertiesMarkerPath(endMarker));
+    m_styleProps.insert("draw:marker-end-width", m_scale*_linePropertiesMarkerScale(endMarker)*(0.1/(strokeWidth*strokeWidth+1)+2.54*strokeWidth));
+  }
+}
+
+const char *libvisio::VSDXContentCollector::_linePropertiesMarkerViewbox(unsigned marker)
+{
+  switch (marker)
+  {
+  case 1:
+  case 2:
+  case 9:
+  case 15:
+    return "0 0 20 10";
+  case 8:
+    return "0 0 20 18";
+  case 3:
+  case 4:
+  case 5:
+  case 6:
+  case 11:
+  case 16:
+  case 17:
+  case 18:
+    return "0 0 20 20";
+  case 12:
+  case 13:
+  case 14:
+    return "0 0 20 30";
+  case 22:
+  case 39:
+    return "0 0 20 40";
+  case 21:
+    return "0 0 30 30";
+  case 10:
+    return "0 0 1131 1131";
+  default:
+    return "0 0 20 30";
+  }
+}
+
+const char *libvisio::VSDXContentCollector::_linePropertiesMarkerPath(unsigned marker)
+{
+  switch (marker)
+  {
+  case 1:
+    return "m10 -4l-14 14l4 4l10 -10l10 10l4 -4z";
+  case 2:
+    return "m10 0-10 10h20z";
+  case 3:
+    return "m10 -8l-14 28l6 3l8 -16l8 16l6 -3z";
+  case 4:
+    return "m10 0-10 20h20z";
+  case 5:
+    return "m10 0-10 20q10,-5 20,0z";
+  case 6:
+    return "m10 0-10 20q10,5 20,0z";
+  case 8:
+    return "m10 0q-2.6,13.4 -10,18q10,-5 20,0q-7.4,-4.6 -10,-18";
+  case 9:
+    return "m-2 -8l4 -4l20 20l-4 4z";
+  case 10: // Copied from what LO exports when using the "circle" marker
+    return "m462 1118-102-29-102-51-93-72-72-93-51-102-29-102-13-105 13-102 29-106 51-102 72-89 93-72 102-50 102-34 106-9 101 9 106 34 98 50 93 72 72 89 51 102 29 106 13 102-13 105-29 102-51 102-72 93-93 72-98 51-106 29-101 13z";
+  case 11:
+    return "m0 0v10h10v-10z";
+  case 12:
+    return "m10 -12l-14 42l9 3l5 -15l5 15l9 -3z";
+  case 13:
+    return "m10 0-10 30h20z";
+  case 14:
+    return "m10 0-10 30h20z m0 12l-5 15h10z";
+  case 15:
+    return "m10 0-10 10h20z m0 3l-5 5h10z";
+  case 16:
+    return "m10 0-10 20h20z m0 7l-5 10h10z";
+  case 17:
+    return "m10 0-10 20q10,-5 20,0z m0 7l-4 8q4,-2 8,0z";
+  case 18:
+    return "m10 0-10 20q10,5 20,0z m0 7l-5 10q5,2 10,0z";
+  case 21:
+    return "m0 0v30h30v-30z m10 10v10h10v-10z";
+  case 22:
+    return "m10 0-10 20l10 20l10 -20z m0 8l-6 12l6 12l6 -12z";
+  case 39:
+    return "m10 0-10 20h20z m0 20-10 20h20z";
+  default:
+    return "m10 0-10 30h20z";
+  }
+}
+
+double libvisio::VSDXContentCollector::_linePropertiesMarkerScale(unsigned marker)
+{
+  switch (marker)
+  {
+  case 11:
+  case 10:
+    return 0.7;
+  case 14:
+  case 15:
+  case 16:
+  case 17:
+  case 18:
+  case 22:
+    return 1.2;
+  default:
+    return 1.0;
   }
 }
 
@@ -481,7 +590,10 @@ void libvisio::VSDXContentCollector::_flushCurrentPath()
     }
     else if (m_currentGeometry[i]["libwpg:path-action"]->getStr() == "M")
     {
-      if ((startX == x && startY == y) && path.count() && !wasMove)
+      if (((startX == x && startY == y)
+           || (m_styleProps["draw:fill"] && m_styleProps["draw:fill"]->getStr() != "none"
+               && m_styleProps["draw:stroke"] && m_styleProps["draw:stroke"]->getStr() == "none"))
+          && path.count() && !wasMove)
       {
         WPXPropertyList closedPath;
         closedPath.insert("libwpg:path-action", "Z");
@@ -501,7 +613,10 @@ void libvisio::VSDXContentCollector::_flushCurrentPath()
     }
     path.append(m_currentGeometry[i]);
   }
-  if ((startX == x && startY == y) && path.count() && !wasMove)
+  if (((startX == x && startY == y)
+       || (m_styleProps["draw:fill"] && m_styleProps["draw:fill"]->getStr() != "none"
+           && m_styleProps["draw:stroke"] && m_styleProps["draw:stroke"]->getStr() == "none"))
+      && path.count() && !wasMove)
   {
     WPXPropertyList closedPath;
     closedPath.insert("libwpg:path-action", "Z");
@@ -518,15 +633,32 @@ void libvisio::VSDXContentCollector::_flushCurrentPath()
 void libvisio::VSDXContentCollector::_flushText()
 {
   if (!m_textStream.size()) return;
+
+  double xmiddle = m_txtxform ? m_txtxform->width / 2.0 : m_xform.width / 2.0;
+  double ymiddle = m_txtxform ? m_txtxform->height / 2.0 : m_xform.height / 2.0;
+
+  transformPoint(xmiddle,ymiddle, m_txtxform);
+
+  double x = xmiddle - (m_txtxform ? m_txtxform->width / 2.0 : m_xform.width / 2.0);
+  double y = ymiddle - (m_txtxform ? m_txtxform->height / 2.0 : m_xform.height / 2.0);
+
   double angle = 0.0;
   transformAngle(angle, m_txtxform);
 
-  double x = 0.0;
-  double y = m_txtxform ? m_txtxform->height: m_xform.height;
-
-  transformPoint(x,y, m_txtxform);
-
   WPXPropertyList textBlockProps;
+
+  bool flipX = false;
+  bool flipY = false;
+  transformFlips(flipX, flipY);
+
+  if (flipX)
+    angle -= M_PI;
+
+  while (angle > M_PI)
+    angle -= 2 * M_PI;
+  while (angle < -M_PI)
+    angle += 2 * M_PI;
+
   textBlockProps.insert("svg:x", m_scale * x);
   textBlockProps.insert("svg:y", m_scale * y);
   textBlockProps.insert("svg:height", m_scale * (m_txtxform ? m_txtxform->height : m_xform.height));
@@ -535,7 +667,7 @@ void libvisio::VSDXContentCollector::_flushText()
   textBlockProps.insert("fo:padding-bottom", m_textBlockStyle.bottomMargin);
   textBlockProps.insert("fo:padding-left", m_textBlockStyle.leftMargin);
   textBlockProps.insert("fo:padding-right", m_textBlockStyle.rightMargin);
-  textBlockProps.insert("libwpg:rotate", -angle*180/M_PI, WPX_GENERIC);
+  textBlockProps.insert("libwpg:rotate", angle*180/M_PI, WPX_GENERIC);
 
   switch (m_textBlockStyle.verticalAlign)
   {
@@ -583,7 +715,7 @@ void libvisio::VSDXContentCollector::_flushText()
   const unsigned char *pTextBuffer = m_textStream.getDataBuffer();
 
   for (std::vector<VSDXParaStyle>::iterator paraIt = m_paraFormats.begin();
-       paraIt < m_paraFormats.end() || charIndex < m_charFormats.size(); paraIt++)
+       paraIt < m_paraFormats.end() || charIndex < m_charFormats.size(); ++paraIt)
   {
     WPXPropertyList paraProps;
 
@@ -595,15 +727,21 @@ void libvisio::VSDXContentCollector::_flushText()
     switch ((*paraIt).align)
     {
     case 0: // left
-      paraProps.insert("fo:text-align", "left");
+      if (!(*paraIt).flags)
+        paraProps.insert("fo:text-align", "left");
+      else
+        paraProps.insert("fo:text-align", "end");
       break;
     case 2: // right
-      paraProps.insert("fo:text-align", "end");
+      if (!(*paraIt).flags)
+        paraProps.insert("fo:text-align", "end");
+      else
+        paraProps.insert("fo:text-align", "left");
       break;
-    case 4: // justify
+    case 3: // justify
       paraProps.insert("fo:text-align", "justify");
       break;
-    case 5: // full
+    case 4: // full
       paraProps.insert("fo:text-align", "full");
       break;
     default: // center
@@ -714,19 +852,43 @@ void libvisio::VSDXContentCollector::_flushText()
 
 void libvisio::VSDXContentCollector::_flushCurrentForeignData()
 {
-  m_currentForeignProps.insert("svg:width", m_scale*m_foreignWidth);
-  m_currentForeignProps.insert("svg:height", m_scale*m_foreignHeight);
-  double x = m_foreignOffsetX;
-  double y = m_foreignOffsetY;
-  transformPoint(x,y);
+  double xmiddle = m_foreignOffsetX + m_foreignWidth / 2.0;
+  double ymiddle = m_foreignOffsetY + m_foreignHeight / 2.0;
 
-  m_currentForeignProps.insert("svg:x", m_scale*x);
-  // Y axis starts at the bottom not top
-  m_currentForeignProps.insert("svg:y", m_scale*(y - m_foreignHeight));
+  transformPoint(xmiddle, ymiddle);
+
+  bool flipX = false;
+  bool flipY = false;
+
+  transformFlips(flipX, flipY);
+
+  WPXPropertyList styleProps(m_styleProps);
+
+  m_currentForeignProps.insert("svg:x", m_scale*(xmiddle - (m_foreignWidth / 2.0)));
+  m_currentForeignProps.insert("svg:width", m_scale*m_foreignWidth);
+  m_currentForeignProps.insert("svg:y", m_scale*(ymiddle - (m_foreignHeight / 2.0)));
+  m_currentForeignProps.insert("svg:height", m_scale*m_foreignHeight);
+
+  double angle = 0.0;
+  transformAngle(angle);
+
+  if (flipX)
+  {
+    m_currentForeignProps.insert("draw:mirror-horizontal", true);
+    angle = M_PI - angle;
+  }
+  if (flipY)
+  {
+    m_currentForeignProps.insert("draw:mirror-vertical", true);
+    angle *= -1.0;
+  }
+
+  if (angle != 0.0)
+    m_currentForeignProps.insert("libwpg:rotate", angle * 180 / M_PI, WPX_GENERIC);
 
   if (m_currentForeignData.size() && m_currentForeignProps["libwpg:mime-type"] && !m_noShow)
   {
-    m_shapeOutputDrawing->addStyle(m_styleProps, WPXPropertyListVector());
+    m_shapeOutputDrawing->addStyle(styleProps, WPXPropertyListVector());
     m_shapeOutputDrawing->addGraphicObject(m_currentForeignProps, m_currentForeignData);
   }
   m_currentForeignData.clear();
@@ -735,10 +897,10 @@ void libvisio::VSDXContentCollector::_flushCurrentForeignData()
 
 void libvisio::VSDXContentCollector::_flushCurrentPage()
 {
-  if (m_pageShapeOrder.size())
+  if (!m_pageShapeOrder.empty())
   {
     std::stack<std::pair<unsigned, VSDXOutputElementList> > groupTextStack;
-    for (std::list<unsigned>::iterator iterList = m_pageShapeOrder.begin(); iterList != m_pageShapeOrder.end(); iterList++)
+    for (std::list<unsigned>::iterator iterList = m_pageShapeOrder.begin(); iterList != m_pageShapeOrder.end(); ++iterList)
     {
       std::map<unsigned, unsigned>::iterator iterGroup = m_groupMemberships.find(*iterList);
       if (iterGroup == m_groupMemberships.end())
@@ -943,7 +1105,7 @@ void libvisio::VSDXContentCollector::collectInfiniteLine(unsigned /* id */, unsi
 
     xmove = points.begin()->first;
     ymove = points.begin()->second;
-    for (std::map<double, double>::iterator iter = points.begin(); iter != points.end(); iter++)
+    for (std::map<double, double>::iterator iter = points.begin(); iter != points.end(); ++iter)
     {
       if (iter->first != xmove || iter->second != ymove)
       {
@@ -1105,6 +1267,7 @@ void libvisio::VSDXContentCollector::_handleForeignData(const WPXBinaryData &bin
   else if (m_foreignType == 2)
   {
     m_currentForeignProps.insert("libwpg:mime-type", "object/ole");
+    m_currentForeignData.append(binaryData);
   }
 }
 
@@ -1122,6 +1285,8 @@ void libvisio::VSDXContentCollector::collectGeometry(unsigned /* id */, unsigned
   if ((m_noFill != noFill) || (m_noLine != noLine) || (m_noShow != noShow) || m_isFirstGeometry)
     _flushCurrentPath();
 
+  _applyLinePattern();
+
   m_isFirstGeometry = false;
   m_noFill = noFill;
   m_noLine = noLine;
@@ -1196,27 +1361,9 @@ void libvisio::VSDXContentCollector::collectArcTo(unsigned /* id */, unsigned le
     double chord = sqrt(pow((y2 - m_y),2) + pow((x2 - m_x),2));
     double radius = (4 * bow * bow + chord * chord) / (8 * fabs(bow));
     int largeArc = fabs(bow) > radius ? 1 : 0;
-    int sweep = bow < 0 ? 1 : 0;
+    bool sweep = (bow < 0);
+    transformFlips(sweep, sweep);
 
-    // If any parent group is flipped, invert sweep
-    unsigned shapeId = m_currentShapeId;
-    while (true)
-    {
-      std::map<unsigned, XForm>::iterator iterX = m_groupXForms.find(shapeId);
-      if (iterX != m_groupXForms.end())
-      {
-        XForm xform = iterX->second;
-        if (xform.flipX) sweep = sweep == 0 ? 1 : 0;
-        if (xform.flipY) sweep = sweep == 0 ? 1 : 0;
-      }
-      else
-        break;
-      std::map<unsigned, unsigned>::iterator iter = m_groupMemberships.find(shapeId);
-      if (iter != m_groupMemberships.end())
-        shapeId = iter->second;
-      else
-        break;
-    }
     m_x = x2;
     m_y = y2;
     arc.insert("svg:rx", m_scale*radius);
@@ -1247,7 +1394,7 @@ void libvisio::VSDXContentCollector::collectNURBSTo(unsigned /* id */, unsigned
 
   // Convert control points to static co-ordinates
   for (std::vector<std::pair<double, double> >::iterator it = controlPoints.begin();
-       it != controlPoints.end(); it++)
+       it != controlPoints.end(); ++it)
   {
     if (xType == 0) // Percentage
       (*it).first *= m_xform.width;
@@ -1269,7 +1416,7 @@ void libvisio::VSDXContentCollector::collectNURBSTo(unsigned /* id */, unsigned
     NURBS.insert("libwpg:path-action", "L");
     double nextX = 0;
     double nextY = 0;
-    double denominator = 0.0000001;
+    double denominator = 1E-10;
 
     for (unsigned p = 0; p < controlPoints.size() && p < weights.size(); p++)
     {
@@ -1291,6 +1438,12 @@ void libvisio::VSDXContentCollector::collectNURBSTo(unsigned /* id */, unsigned
   m_x = x2;
   m_y = y2;
   transformPoint(m_x, m_y);
+  NURBS.clear();
+  NURBS.insert("libwpg:path-action", "L");
+  NURBS.insert("svg:x", m_scale*m_x);
+  NURBS.insert("svg:y", m_scale*m_y);
+  m_currentGeometry.push_back(NURBS);
+
 }
 
 double libvisio::VSDXContentCollector::_NURBSBasis(unsigned knot, unsigned degree, double point, const std::vector<double> &knotVector)
@@ -1531,6 +1684,37 @@ void libvisio::VSDXContentCollector::transformAngle(double &angle, XForm *txtxfo
   angle = fmod(2.0*M_PI + (y1 > y0 ? 1.0 : -1.0)*acos((x1-x0) / sqrt((x1-x0)*(x1-x0) + (y1-y0)*(y1-y0))), 2.0*M_PI);
 }
 
+void libvisio::VSDXContentCollector::transformFlips(bool &flipX, bool &flipY)
+{
+  if (!m_isShapeStarted)
+    return;
+
+  if (!m_currentShapeId)
+    return;
+
+  unsigned shapeId = m_currentShapeId;
+
+  while (true)
+  {
+    std::map<unsigned, XForm>::iterator iterX = m_groupXForms.find(shapeId);
+    if (iterX != m_groupXForms.end())
+    {
+      XForm xform = iterX->second;
+      if (xform.flipX)
+        flipX = !flipX;
+      if (xform.flipY)
+        flipY = !flipY;
+    }
+    else
+      break;
+    std::map<unsigned, unsigned>::iterator iter = m_groupMemberships.find(shapeId);
+    if (iter != m_groupMemberships.end())
+      shapeId = iter->second;
+    else
+      break;
+  }
+}
+
 void libvisio::VSDXContentCollector::collectShapeId(unsigned /* id */, unsigned level, unsigned /* shapeId */)
 {
   _handleLevelChange(level);
@@ -1646,6 +1830,7 @@ void libvisio::VSDXContentCollector::collectShape(unsigned id, unsigned level, u
         m_foreignOffsetY = m_stencilShape->m_foreign->offsetY;
         m_foreignWidth = m_stencilShape->m_foreign->width;
         m_foreignHeight = m_stencilShape->m_foreign->height;
+        m_currentForeignData.clear();
         _handleForeignData(m_stencilShape->m_foreign->data);
       }
 
@@ -1784,10 +1969,10 @@ void libvisio::VSDXContentCollector::collectText(unsigned /*id*/, unsigned level
 }
 
 void libvisio::VSDXContentCollector::collectVSDXParaStyle(unsigned /* id */ , unsigned level, unsigned charCount, double indFirst, double indLeft, double indRight,
-    double spLine, double spBefore, double spAfter, unsigned char align)
+    double spLine, double spBefore, double spAfter, unsigned char align, unsigned flags)
 {
   _handleLevelChange(level);
-  VSDXParaStyle format(charCount, indFirst, indLeft, indRight, spLine, spBefore, spAfter, align);
+  VSDXParaStyle format(charCount, indFirst, indLeft, indRight, spLine, spBefore, spAfter, align, flags);
   m_paraFormats.push_back(format);
 }
 
@@ -1875,7 +2060,7 @@ void libvisio::VSDXContentCollector::collectCharIXStyle(unsigned /*id*/ , unsign
 }
 
 void libvisio::VSDXContentCollector::collectParaIXStyle(unsigned /* id */, unsigned level, unsigned /* charCount */, double /* indFirst */, double /* indLeft */, double /* indRight */,
-    double /* spLine */, double /* spBefore */, double /* spAfter */, unsigned char /* align */)
+    double /* spLine */, double /* spBefore */, double /* spAfter */, unsigned char /* align */, unsigned /* flags */)
 {
   _handleLevelChange(level);
 }
@@ -1984,7 +2169,7 @@ void libvisio::VSDXContentCollector::_handleLevelChange(unsigned level)
         m_NURBSData = m_stencilShape->m_nurbsData;
         m_polylineData = m_stencilShape->m_polylineData;
 
-        if (m_currentGeometry.size() == 0)
+        if (m_currentGeometry.empty())
         {
           for (unsigned i = 0; i < m_stencilShape->m_geometries.size(); i++)
           {
